/********************************************************
 *Class:       Projectile.cpp
 *Author:      Peter Asplund
 *Created:     2004-11-25
 *Description: Calculates the trajectory of an object.
 *Arguments:   Initial speed and angle (in degrees)
 ********************************************************/

#include <math.h>
#include "include/projectile.h"
#include <vector>
#include <iostream>

using namespace std;

/**********************************************
 *Function:    Default constructor
 *Arguments:   None
 *Returns:     NULL
 *Description: Initiates one trajectory object
 *             destined to fall!
 **********************************************/
Projectile::Projectile(): yPos(0), xPos(0), oldXPos(0), initialSpeed(0), speed(0), xSpeed(0), theta(0) { 
 
}

/**********************************************
 *Function:    Constructor
 *Arguments:   double _speed
 *             double _theta
 *Returns:     NULL
 *Description: Initiates one trajectory object.
 *             Sets intial speed to _speed, and
 *             angle of fire to _theta-degrees.
 **********************************************/
Projectile::Projectile(double _speed, double _theta): yPos(0), xPos(0), oldXPos(0) {
  
  initialSpeed = _speed;
  speed = _speed;
  theta = _theta*M_PI/180.0;
  xSpeed = speed*cos(theta);
  ySpeed = speed*sin(theta);

}

/*******************************************
 *Function:   getSpeed
 *Arguments:  NULL
 *Returns:     The speed of the projectile
 *Description:Speed is in m/s
 ******************************************/
double Projectile::getSpeed() const {

  return xSpeed;

}

/*******************************************
 *Function:   setPosition
 *Arguments:  int y (position in y)
 *Returns:    NULL
 *Description:Sets the y-position of the object
 *            to zero (ie. resetting it)
 ******************************************/
void Projectile::setPosition(int y) {

  xPos = y;
  speed = y;

}

/*******************************************
 *Function:   getXPos
 *Arguments:  int* stepSize
 *Returns:    The x-position of the projectile
 *Description:Calculates the x-position by using
 *            Euler approximation. Depends on old
 *            x-position.
 ******************************************/

double Projectile::getXPos(double* stepSize) {

  /************************
   *Euler to get xPosition
   ***********************/
  xPos += (*stepSize) * xSpeed;

  /**********************************
   *Euler to let gravity do it's shit
   **********************************/
  ySpeed += (*stepSize) * g;

  speed = xSpeed + ySpeed; //Sum of x- and y-speed
  return xPos;

}

/*******************************************
 *Function:   getPos
 *Arguments:  int* stepSize
 *Returns:    The x-position of the projectile
 *Description:Calculates the x-position by using
 *            Euler approximation. Depends on old
 *            x-position.
 ******************************************/
void Projectile::getPos(double* stepSize, vector<double> & positions) {

  xPos = getXPos(stepSize);

  //Formula to get y-position from x-position
  yPos = xPos * tan(theta) + (g*xPos*xPos) / (2 * initialSpeed * initialSpeed * cos(theta) * cos(theta));

  //return vector
  positions.at(0) = xPos;
  positions.at(1) = yPos;

}

Projectile::~Projectile() {
  //Destruktor
}
