/*******************************************
 *File: tank.cpp
 *Author: Jonas Hellsten
 *Created: 2004-11-14
 *Description: Simulates the dynamics of a tank
 *Dependent on Irrlicht, engine, suspension.
 ******************************************/


#include <vector>
#include <iostream>

#ifdef WIN32
#include "irrlicht.h"
#endif // WIN32
#ifdef linux
#include "irrlicht/irrlicht.h"
#endif // LINUX

#include "include/tank.h"

using namespace irr;
using namespace core;




Tank::~Tank(){
  //destruktor   
}    
/******************************************
   *Function: various set functions
   *Description: Setts the coresponding boolean to true/false to show if the buton has been pressed or not.
   *****************************************/
void Tank::accelerate(bool keyPressed){
    isAccelerating = keyPressed;
}
void Tank::breaking(bool keyPressed){
    isBreaking = keyPressed;
}
void Tank::turnRight(bool keyPressed){
    
    isTurningRight = keyPressed;
}
void Tank::turnLeft(bool keyPressed){
    isTurningLeft = keyPressed;
}
void Tank::setGearUp(){
    motor.setGearUp();
}
void Tank::setGearDown(){
    motor.setGearDown();
}
void Tank::turnTurretRight(bool keyPressed){
    torn.rotateRight(keyPressed);
}
void Tank::turnTurretLeft(bool keyPressed){
    torn.rotateLeft(keyPressed);
}
void Tank::elevateTurretUpp(bool keyPressed){
    torn.elevate(keyPressed);
}
void Tank::elevateTurretDown(bool keyPressed){
    torn.dump(keyPressed);
}

void Tank::shoot(bool keyPressed){
    torn.shoot(keyPressed);
}
void Tank::pause(){
    paused = !paused;
}
void Tank::jump(){
    jumping = rand()%14;
}
/******************************************
  *Function: reset
  *Description: Resets the tank to the origo position with
  *   Zero velocity and rotation
  ******************************************/  
void Tank::reset(){
    angularVelocity = vector3df(0,0,0);
    velocity = vector3df(0,0,0);
    tankNodeVelocity = vector3df(0,0,0);
    tankNode->setPosition(vector3df(0,0,0));
    chassi->setPosition(vector3df(0.0f, 0.0f, 0.0f));
    tankNode->setRotation(vector3df(0.0f, 0.0f, 0.0f));
    chassi->setRotation(vector3df(0.0f, 0.0f, 0.0f));
    motor = engine::Engine();
    
}  
//rotates the camera around the tank
void Tank::rotateCamera(float x, float y){
    if(cameraNode != 0){
        vector3df cameraRotation = cameraNode->getRotation();
        float sensetivity = 1; //knslighet
        cameraRotation.X = (x*sensetivity-cameraX);
        cameraRotation.Y = (y*sensetivity-cameraY);
        cameraNode->setRotation(cameraRotation);
        //cout <<"x: "<< x <<"\ty: " << y <<endl;
        //if(cursor != 0)
        //cursor->setPosition(0.5f, 0.5f);
    } 
      
}
    

/******************************************
  *Function: calculatePosition
  *Receives: A stepsize for the diferential equations. 
    *Normaly this will be the time it takes to render one frame
  *Returns: true if sucessfull
  *Description: Calculates the new position and direction of the tank
  ******************************************/  
bool Tank::calculatePosition(float frameTime){
    
    //do nothing if paused
    if(paused)
        return true;
        
    vector3df tankPosition = tankNode->getPosition();
    //do the calculations for the engine
    motor.calculateAcceleration(isAccelerating, isBreaking, frameTime);
    
    //do the calculations for the turret
    torn.calculate(frameTime);
    
    /*calculate the propultion separate from the force-equations as the
     velocity of the tank always is applied in the direction of the tank.
     As we dont take friction into account it would feel as it was driving on ice.
     */
    float tankSpeed = motor.getSpeed();
    vector3df direction = getDirection();
    
    tankPosition.X += direction.X*tankSpeed * frameTime;
    tankPosition.Z -= direction.Z*tankSpeed * frameTime;
    
    /*there is boundaries the tank must be within
    this to avoid going into the mountains
    must chech if outside these boundaries,
    and if so, change the position to the other side.*/
    //cout <<tankPosition.Z <<"\t" <<tankPosition.X <<endl;
    
    float limit = 400;
    if (tankPosition.X > limit)
        tankPosition.X = tankPosition.X - limit;
    else if (tankPosition.X < -1*limit)
        tankPosition.X = limit + tankPosition.X;
    if (tankPosition.Z > limit){
        tankPosition.Z =  tankPosition.Z - limit;
    }    
    else if (tankPosition.Z < -1*limit)
        tankPosition.Z = limit + tankPosition.Z;
    
    tankPosition += tankNodeVelocity * frameTime;
    
    
    tankNode->setPosition(tankPosition);
    
    //calculate the translating forces
    vector3df resultingForces;
    //calculate the spring forces and add to the resulting translating force
    float sf[14] ;
    for(int i = 0;i < 14;i++){
        sf[i] = spring[i]->calculateSpringForce(springPoints[i]->getAbsolutePosition().Y, frameTime);
        if(i == jumping ){
            //printf("jumping:%d\tsf:%f\n", jumping,sf[jumping] );
            sf[jumping] = sf[jumping] + 100000000;
            jumping = -1;
        }     
        resultingForces.Y += sf[i];
    }
    //apply gravity force 
    resultingForces.Y += CHASSIMASS * GRAVITY;
    
    //integrate translating resulting force to position
    velocity.X += resultingForces.X/CHASSIMASS * frameTime;
    velocity.Y += resultingForces.Y/CHASSIMASS * frameTime;
    velocity.Z += resultingForces.Z/CHASSIMASS * frameTime;
    
    //get the position from irrlicht
    vector3df chassiPosition = chassi->getPosition();
    chassiPosition.X += velocity.X * frameTime;
    chassiPosition.Y += velocity.Y * frameTime;
    chassiPosition.Z += velocity.Z * frameTime;
    
    
    
    
    //uppdate the position to irrlicht
    chassi->setPosition(chassiPosition);
    
    //Begin to calculate the moment forces
    vector3df momentForces;
    //calculate the centripetal force that act sideways then you turn
    /***********************
    r = v / omega
    v = omega *r 
    force = (v^2 / r) * Mass
    ************************/
    float centripetalForce = -1*CHASSIMASS * motor.getSpeed()* angularVelocity.Y * 0.02 ;//added constant to decrease the effect.
    
    //add the centripetal force (only in Z-direction)
    momentForces.Z += centripetalForce * (springPoints[3]->getPosition().Y+springPoints[10]->getPosition().Y)*(-0.5); //average the height of springpoints index 3,10
    //add force from engine (only in X-direction)
    momentForces.X +=motor.getForce()*(springPoints[3]->getPosition().Y+springPoints[10]->getPosition().Y)*(-0.5);
    //add forces from the springs (in Y/X-direction)
    for (int i = 0; i<14;i++){
       momentForces.X += sf[i]*(-1)*springPoints[i]->getPosition().Z;
       momentForces.Z += sf[i]*springPoints[i]->getPosition().X;
   }
   //add forces then you turn (only in Y-direction)
   float turningMoment = 0;
    if( isTurningRight != isTurningLeft)
     switch(motor.getGear()){
        case -2:
        turningMoment = 15000000;
        break;
        case -1:
        turningMoment = 10000000;
        break;
        case 0:
        turningMoment = 48000000;
        break;
        case 1:
        turningMoment = 10000000;
        break;
        case 2:
        turningMoment = 15000000;
        break;
        case 3:
        turningMoment = 24000000;
        break;
        case 4:
        turningMoment = 28000000;
        break;
     }
     
    if(isTurningLeft == true){ //to make sure you turn in the right direction
        turningMoment = -turningMoment;   
    }
    momentForces.Y = (turningMoment -angularVelocity.Y * 1000000)/3; //constant added to get realistic result
    
    //integrate the moment forces twice to get rotation
    angularVelocity.X += (momentForces.X / IX) * frameTime;
    angularVelocity.Y += (momentForces.Y / IY) * frameTime;
    angularVelocity.Z += (momentForces.Z / IZ) * frameTime;
    vector3df chassiRotation = chassi->getRotation(); 
    chassiRotation.X += angularVelocity.X * frameTime;
    chassiRotation.Y += 0; //se coment below
    chassiRotation.Z += angularVelocity.Z * frameTime;
    chassi->setRotation(chassiRotation);
    
    /*As you only want to drive forward, not up and down, 
    rotations in in Y-axle is handled diffrent from the rest of the tank
    we apply rotation in X and Z to the chassi, and Y to the tankNode.
    The point being that this may have helped then doing colision detection 
    and driving upp hills*/
    
    vector3df tankRotation = tankNode->getRotation();
    tankRotation.Y += angularVelocity.Y * frameTime;
    tankNode->setRotation(tankRotation);
    
    
    
   //calculate new positions for the wheels (to keep them to the ground)
   for (int i = 0; i<14;i++){
       float tmp = (springPoints[i]->getAbsolutePosition().Y-0.131657)/cos((-1)*chassiRotation.X*2*PI/360);
       vector3df wheelpos = wheel[i]->getPosition();
       wheelpos.Y = -tmp;
       wheel[i]->setPosition(wheelpos);
   }

    camera->setTarget(tankNode->getPosition()); 
}

/******************************************
  *Function: getDirection
  *Returns: Direction of the tank.
  *Description: Rotates a vector the same way as the tank.
  ******************************************/  
vector3df Tank::getDirection(){
    vector3df tankRotation = tankNode->getRotation();
    vector3df direction = vector3df(0,0,1);
    matrix4 rotationMatrix = matrix4();
    rotationMatrix.setRotationDegrees(tankRotation);
    rotationMatrix.rotateVect(direction);
    return direction; 
}

/******************************************
  *Function: Tank (Constructor)
  *Parameters: Takes in the device node and the master node of the tank
  *Description: Create the Tank object at origo, reads the models and so on.
  ******************************************/  
Tank::Tank(IrrlichtDevice* device, scene::ISceneNode* tankNode2){ 
    tankNode = tankNode2;
    //creates the engine
    motor = engine::Engine();
    motor.setGearUp();
    
    cameraX = 0;
    cameraY = 0;
    
        //Initiates all positions
      angularVelocity = vector3df(0,0,0);
      velocity = vector3df(0,0,0);
      tankNodeVelocity = vector3df(0,0,0);
      
      //sett all boolaens to false
      turretTurningRight = false;
      turretTurningLeft = false;
      turretUpp = false;
      turretDown = false;
      isAccelerating = false;
      isBreaking = false;
      isTurningRight = false;
      isTurningLeft = false;
      paused = true;
      jumping = -1;
    
    //get a pointer to the irrlicht stuff
     video::IVideoDriver* driver = device->getVideoDriver();
    scene::ISceneManager* smgr = device->getSceneManager();
    gui::ICursorControl* cursor = device->getCursorControl();
    cursor->setPosition(0.5f, 0.5f);
    
    //loading chassi
    chassi =  smgr->addAnimatedMeshSceneNode(
    smgr->getMesh("../media/tank/chassi.3ds"));
    chassi->setScale(vector3df(1.0f, 1.0f, 1.0f));
    chassi->setMaterialFlag(video::EMF_LIGHTING, false);
    chassi->setMaterialTexture(0, driver->getTexture("../media/tank/kamo.jpg"));
    tankNode->addChild(chassi);
    chassi->setPosition(vector3df(0.0f, 0.0f, 0.0f));
    
    
    
    //loading camera
    cameraNode = smgr->addEmptySceneNode();
    chassi->addChild(cameraNode);
    camera = smgr->addCameraSceneNode(0,vector3df(0,0,-10), tankNode->getAbsolutePosition());
    cameraNode->addChild(camera);
    camera->setFarValue(1000.0f);
    
   
    
    /*create the animation controller class for the turret and gun*/
    torn = turret::Turret(device, chassi);
    
    //load tracks
    for(int i = 0; i<2; i++){
        track[i] =  smgr->addAnimatedMeshSceneNode(
        smgr->getMesh("../media/tank/track.3ds"));
        track[i]->setMaterialFlag(video::EMF_LIGHTING, false);
        track[i]->setMaterialTexture(0, driver->getTexture("../media/tank/rust3.jpg"));
        tankNode->addChild(track[i]);
        track[i]->setScale(vector3df(1.0f, 1.1f, 1.15f));
        track[i]->setPosition(vector3df((-1+2*i)*1.7f , 0.6f, 0.05));
    }
    
    //load the wheels
    scene::ISceneNode* wheelNode = smgr->addEmptySceneNode();
    chassi->addChild(wheelNode);
    scene::IAnimatedMesh* wheelModel = smgr->getMesh("../media/tank/wheel.3ds");
    //scene::IAnimatedMesh* cone = smgr->getMesh("../media/tank/ball.3ds");
    for(int i = 0; i<14; i++){
        springPoints[i] = smgr->addEmptySceneNode();
        wheelNode->addChild(springPoints[i]);

        wheel[i] = smgr->addAnimatedMeshSceneNode(wheelModel);
        wheel[i]->setMaterialFlag(video::EMF_LIGHTING, false);
        wheel[i]->setMaterialTexture(0, driver->getTexture("../media/tank/metalox.jpg"));
        springPoints[i]->addChild(wheel[i]);

        wheel[i]->setPosition(vector3df(0.0f, -.75f, 0.0f));
        //printf("Starting length: %f\t absolute: %f\n", wheel[i]->getPosition().Y*(-1), wheel[i]->getAbsolutePosition().Y);
        spring[i] = new suspension::Suspension(wheel[i]->getPosition().Y*(-1));
        
        if(i<7)
            springPoints[i]->setPosition(vector3df(-1.7f , 1.0f, 2.7f-0.9*i));  
        if(i>=7)
            springPoints[i]->setPosition(vector3df(1.7f , 1.0f, 2.7f-0.9*(i-7)));
    }
    
    //initiates a random generator for the jump function
    srand( time(NULL) ); 
 
    
}

