/*******************************************
 *File: tank.h
 *Author: Jonas Hellsten
 *Created: 2004-11-14
 *Description: Simulates the dynamics of a tank
 *Dependent on Irrlicht, engine, suspension.
 ******************************************/
 
#ifndef TANK_H
#define TANK_H

#include "engine.h"
#include "suspension.h"
#include "turret.h"

#include <vector>

using namespace irr;
using namespace core;

class Tank 
{
    public:
        Tank(IrrlichtDevice* device, scene::ISceneNode* tankNode);
        ~Tank();
        
        //the engine
        engine::Engine motor;
        //the turret
        turret::Turret torn;
        
        //Functions
        bool calculatePosition(float frametime);
        vector3df getDirection();
        void accelerate(bool keyPressed);
        void breaking(bool keyPressed);
        void turnRight(bool keyPressed);
        void turnLeft(bool keyPressed);
        void setGearUp();
        void setGearDown();
        void turnTurretRight(bool keyPressed);
        void turnTurretLeft(bool keyPressed);
        void elevateTurretUpp(bool keyPressed);
        void elevateTurretDown(bool keyPressed);
        void shoot(bool keyPressed);
        void rotateCamera(float x, float y);
        void pause();
        void reset();
        void jump();
            
        //SceneNodes
        scene::ISceneNode* tankNode;
         scene::ISceneNode* springPoints[14];
         scene::ISceneNode* chassi;
         scene::ISceneNode* wheel[14];
         scene::ISceneNode* track[2];
         scene::ISceneNode* cameraNode;
         scene::ICameraSceneNode* camera;
         //the cursor controller node
         gui::ICursorControl* cursor;
         
    private:
        
        //vissa variabler
        vector3df angularVelocity;
        vector3df velocity;
        vector3df tankNodeVelocity;
        suspension::Suspension* spring[14];
        
        //boolean parameters
        bool isAccelerating ;
        bool isBreaking ;
        bool isTurningRight ;
        bool isTurningLeft ;
        bool turretTurningRight;
        bool turretTurningLeft;
        bool turretUpp;
        bool turretDown;
        bool paused;
        int jumping;
        
        float cameraX, cameraY; 
        
        //constants 
        static const float IX = 381250/10;
        static const float IY = 68450;
        static const float IZ = 327200/10;
        static const float CHASSIMASS = 60000;
        static const float GRAVITY = 9.82;

};
#endif //TANK_H

