/*  
*   Tanxim 2k5
*   by Peter Asplund, Henrik Hedman, Jonas Hellsten
*   Tanksimulator in modelering och simulering at linkpings university
*/
/***********************************************************
 *File: main.cpp
 *Created: 2004-11-03
 *Modified: 2004-12-16
 *Description: Initiates the graphics engine
 *             Key listener
 *             Creates the static objects in the scene
 *             The main rendering loop
 **********************************************************/



//include namespaces
#include "iostream"
#include <wchar.h>

#ifdef WIN32
#include "irrlicht.h"
#endif // WIN32
#ifdef linux
#include "irrlicht/irrlicht.h"
#endif // Linux

#include <math.h>
#include <stdio.h>
#include "include/tank.h"

             
// Irrlicht Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;


    //gloabal variables
   IrrlichtDevice* device = 0;
   bool quit = false;

   Tank* tank = NULL;
   
class MyEventReceiver : public IEventReceiver
{
    public:
    virtual bool OnEvent(SEvent event){
        
        if (tank != 0 && event.EventType == irr::EET_MOUSE_INPUT_EVENT){
            if (event.MouseInput.Event == EMIE_MOUSE_MOVED) {
                tank->rotateCamera(event.MouseInput.Y,event.MouseInput.X);
            }     
        }    
        
        if (tank != 0 && event.EventType == irr::EET_KEY_INPUT_EVENT){
            
            switch(event.KeyInput.Key){
                case KEY_KEY_W:
                    tank->accelerate(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_S:
                    tank->breaking(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_A:
                    tank->turnLeft(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_D:
                    tank->turnRight(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_Q:
                    if(event.KeyInput.PressedDown)
                        tank->setGearUp();
                break;
                case KEY_KEY_E:
                    if(event.KeyInput.PressedDown)
                        tank->setGearDown();
                break;
                case KEY_KEY_J :
                    tank->turnTurretLeft(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_L :
                    tank->turnTurretRight(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_I :
                    tank->elevateTurretUpp(event.KeyInput.PressedDown);
                break;
                 case KEY_KEY_K :
                    tank->elevateTurretDown(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_V :
                    tank->shoot(event.KeyInput.PressedDown);
                break;
                case KEY_KEY_R :
                    if(event.KeyInput.PressedDown)
                        tank->reset();
                break;
                case KEY_SPACE :
                    if(event.KeyInput.PressedDown)
                        tank->pause();
                break;
                 case KEY_KEY_C :
                    if(event.KeyInput.PressedDown)
                        tank->jump();
                break;
                
                case KEY_ESCAPE:
                    quit = true;
                default:
                break;
            return true;
            }    
        }
    } 
};       

int main()
{
    //create eventreciever and an irrlicht device
    MyEventReceiver receiver;
    device = createDevice(video::EDT_OPENGL, core::dimension2d<s32>(1024,768),
    32, true, false, false, &receiver);
    
    video::IVideoDriver* driver = device->getVideoDriver();
    scene::ISceneManager* smgr = device->getSceneManager();
    
    //Root Node for the tank
    ISceneNode* tankNode = smgr->addEmptySceneNode();
    tankNode->setPosition(vector3df(0.0f, 0.0f, 0.0f));
    tank = new Tank(device, tankNode);
    

    //Creates the ground
    scene::ISceneNode* ground = smgr->addAnimatedMeshSceneNode(
    smgr->getMesh("../media/env/mark2.3ds"));
    ground->setMaterialFlag(video::EMF_LIGHTING, false);
    ground->setMaterialTexture(0, driver->getTexture("../media/env/sand3.jpg"));
    ground->setScale(vector3df(10.0f, 10.0f, 10.0f));
    
      //creates skybox
    scene::ISceneNode* skybox = smgr->addSkyBoxSceneNode(
		driver->getTexture("../media/skybox/dream_up.jpg"),
		driver->getTexture("../media/skybox/dream_dn.jpg"),
		driver->getTexture("../media/skybox/dream_rt.jpg"),
		driver->getTexture("../media/skybox/dream_lt.jpg"),
		driver->getTexture("../media/skybox/dream_ft.jpg"),
		driver->getTexture("../media/skybox/dream_bk.jpg"));  

    //Create boxes with 10m distance to visualise speed
    int stolpNo = 50;
    scene::ISceneNode* stolpar[stolpNo];
    for(int i=0;i<stolpNo; i++){
        stolpar[i] = smgr->addTestSceneNode(1.0f);
        stolpar[i]->setPosition(vector3df(5,1,10*i));
        stolpar[i]->setMaterialTexture(0, driver->getTexture("../media/brickwall.jpg"));
    }    
   
   gui::IGUIFont* font = device->getGUIEnvironment()->getFont("../media/fonthaettenschweiler.bmp");
    
    device->getCursorControl()->setVisible(true);
    int fps = 0;        
    int lastFPS = -1;
    while(device->run())
    {      
        //draw scene and print the gear and speed to the screen
        driver->beginScene(true, true, video::SColor(250, 90, 90, 156));
        smgr->drawAll();
        wchar_t fonttext[100];
      	swprintf(fonttext, 100, L"Gear: %d | Speed: %.2f km/h | FPS: %d",
              tank->motor.getGear(),tank->motor.getSpeed()*3.6,fps);
        font->draw(fonttext, core::rect<s32>(10,10,300,50), video::SColor(255,255,255,255));
        driver->endScene();
        fps = driver->getFPS();
        if(lastFPS != fps)
        {
                wchar_t tmp[1024];
                swprintf(tmp, 1024, L"Tanxim 2k5 | (%s) (fps: %d)",
                    driver->getName(), fps); 
                device->setWindowCaption(tmp);
                lastFPS = fps;
        }
        //calculate the position and rotation of the tank
        if(fps != 0){   
                tank->calculatePosition(1/float(fps));
        }
        
        if (quit)
            break;
    }
    device->drop();
    return 0;
}

                        
                            

