/*******************************************
 *File: turret.h
 *Author: Jonas Hellsten
 *Created: 2004-12-12
 *Description: Simulates the turret of a tank
 *
 ******************************************/

#include "include/turret.h"
#include <math.h>
#include <vector>
#include <iostream>




using namespace std;
using namespace irr;

namespace turret {
/****
Empty Constructor
*****/
Turret::Turret(){
}    
/******************************************
  *Function: Contructor
  *Parameters: The Irrlicht device node to be able to load objects
                The Scene Node for the chassi to be able to link the objects to the chassi.
  *Description: Creates the objects of the turret and attatches these to the tank.
  ******************************************/  
Turret::Turret(IrrlichtDevice* device, scene::ISceneNode* chassi2){
    chassi = chassi2;
    
    //get a pointer to the irrlicht stuff
     video::IVideoDriver* driver = device->getVideoDriver();
    scene::ISceneManager* smgr = device->getSceneManager();
    
    //Loading turret
    turret =  smgr->addAnimatedMeshSceneNode(
    smgr->getMesh("../media/tank/turret.3ds"));
    turret->setMaterialFlag(video::EMF_LIGHTING, false);
    turret->setMaterialTexture(0, driver->getTexture("../media/tank/kamo.jpg"));
    chassi->addChild(turret);
    turret->setPosition(vector3df(0.0f, 2.0f, 0.0f));
    
    //loading an empty sceneNode to make the gun rotate around the right point
    canonNode = smgr->addEmptySceneNode();
    turret->addChild(canonNode);
    canonNode->setPosition(vector3df(0.0f, 0.4f, 2.3f));
    
     //loading the gun
    canon =  smgr->addAnimatedMeshSceneNode(
    smgr->getMesh("../media/tank/canon.3ds"));
    canon->setMaterialFlag(video::EMF_LIGHTING, false);
    canon->setMaterialTexture(0, driver->getTexture("../media/tank/kamo.jpg"));
    canonNode->addChild(canon);
    canon->setPosition(vector3df(0.0f, 0.0f, 2.2f));
    
    float angularVelocity = 0;
	
	bool rotatingRight = false;
	bool rotatingLeft = false;
	bool elevating = false;
	bool dumping = false;
	float shootStrength = 0;
}    
/** Destructor **/
Turret::~Turret(){
}    
/*****
Functions to set booleans in the class
*****/    
void Turret::rotateRight(bool rotateRight){
    rotatingRight = rotateRight;
}    
void Turret::rotateLeft(bool rotateLeft){
    rotatingLeft = rotateLeft;
}    
void Turret::elevate(bool elevate){
    elevating = elevate;
}    
void Turret::dump(bool dump){
    dumping = dump;
}
/****
Funcion: Shoot
Parameters: shoot, describes if the shoot button is pressed or not.
Description: Measures the time the button is pressed, this is not an accurate measurement.
            Then the shoot variable reverts from true to false the canon will shoot.
*******/
void Turret::shoot(bool shoot){
    if(!shoot && shootStrength > 0)
        shooting = true;
    if(shoot)
        shootStrength += 10;
}

/****
Funcion: calculate
Parameters: frameTime, the time it takes to render one frame, used as the stepsize.
Description: calculates the new rotation of the turret and the canon. 
            Also handels the shooting of the gun if the shooting varable is true.
*******/
void Turret::calculate(float frameTime){
    
    //calculate the new rotational speed for the turret
    float moment;
    if(rotatingRight && !rotatingLeft)
        moment = -1*MOMENT;
    if(!rotatingRight && rotatingLeft)
        moment = MOMENT;
        
    angularVelocity += (moment - angularVelocity * 500000)/I * frameTime;
    //integrate to get rotation and apply
    vector3df turretRotation = turret->getRotation(); 
    turretRotation.Y += angularVelocity * frameTime; 
    turret->setRotation(turretRotation);

    //calculate new elevation for the gun
    vector3df gunRotation = canonNode->getRotation(); 

    if (elevating && !dumping)
        gunRotation.X -= ELEVATIONSPEED * frameTime;
    if (!elevating && dumping)
        gunRotation.X += ELEVATIONSPEED * frameTime;

    //check if it has rotated out of its limits
    if (gunRotation.X < ELEVATIONLIMIT)
        gunRotation.X = ELEVATIONLIMIT;
    if (gunRotation.X > DUMPLIMIT)
        gunRotation.X = DUMPLIMIT;

    
    canonNode->setRotation(gunRotation);
    
    if ( shooting ){
        /*****************************
        
        
        Shoot bullet away here !!!
        !!!
        to be added soon!!!
        
        
        *********************************/
        
        shootStrength = -1;
        shooting = false;
    }
        
    
}    
}

    
